

//////////////////////////////////////////////////////////////////
//                                                              //
//           PLINK (c) 2005-2006 Shaun Purcell                  //
//                                                              //
// This file is distributed under the GNU General Public        //
// License, Version 2.  Please see the file COPYING for more    //
// details                                                      //
//                                                              //
//////////////////////////////////////////////////////////////////


#ifndef __MODEL_H__
#define __MODEL_H__

#include<vector>
#include "plink.h"

using namespace std;

class Model {
  
 public:
  
  Model();
  void setPlink(Plink * p_) { P=p_; }
 
  void setMissing();

  void addAdditiveSNP(CSNP*);
  void addDominanceSNP(CSNP*);
  void addCovariate(int);
  void addInteraction(int,int);
  void buildDesignMatrix();
  vector<bool> validParameters();
  bool isValid() { return all_valid; }
  double getStatistic();
  double linearHypothesis(matrix_t &, vector_t &);
  int Ysize() { return nind; }

  vector<string> label;
  int testParameter;

 protected:
 
  Plink * P;

  // Independent variable
  matrix_t X;

  // Missing flag
  vector<bool> miss;

  int nind;
  int np;  // Main effects + interaction + intercept

  vector<bool> valid;
  bool all_valid;

  vector_t coef;  // beta
  matrix_t S;     // Sigma


  // List of additive SNP effects
  // assuming SNP major mode

  vector<CSNP*> additive;

 
  // List of dominance deviation SNP effects

  vector<CSNP*> dominance;

  // List of covariates (clist)

  vector<int> covariate;

  // List of pairwise interactions
  // ( indexing previously specified components, 1,2,..)
  
  vector<int2> interaction;
  
  
};


#endif
